/* vRouter netReader
 *  Purpose: Parses a network-descriptor file.  
 *           Populate a network object.
 *
 *
 *  CHANGES:
 *
 *   0.02 Initial Release (bscholl+@cmu.edu)
 *
*/

#include<iostream.h>
#include<string>

#include "netreader.h"
#include "ipaddress.h"
#include "router.h"
#include "packet.h"
#include "flow.h"
#include "parse.h"

#include <vector>
#include <sys/time.h>


/* 
 * Construct  */
Netreader::Netreader()
{
 net = new Network;
}

/* 
 * Populate a network object from a stream 
 */ 
Network  *Netreader::read(istream &s)
{
  

  net->clear();
  parse(s);
  return net;
}

/* 
 * Main parsing routine 
 */
void Netreader::parse(istream &in)
{
  string token;

  token = getString(in);

  Router * rtr = NULL;
  DataLink * dl = NULL;

  while(token != "end-network")
    {
      if(token == "router")
	{
	  rtr = parse_router(in);
	  if(rtr)  net->addRouter(rtr); 
	}
      else if (token == "datalink")
	{
	  dl = parse_datalink(in);
	  if(dl)  net->addDataLink(dl);
	}
      else
	{
	  cerr << " ** Parse Error on Token [" << token << "]" << endl;
	  return;
	}
      token = getString(in);
    }
}

/* 
 * Parse a router structure from the input stream and return a pointer to it
 * If something goes wrong, output error info to cerr and return NULL
 */ 

Router * Netreader::parse_router(istream &in)
{
  string name;
  name = getString(in);
 
  Router * theRouter = new Router(name);

  string token;
  token = getString(in);

  while(token != "end-router")
    {
        if(token == "interface")
	{
	  string network_address;
	  string IP_address;
	  network_address = getString(in);
	  IP_address = getString(in);

	  DataLink * DL = net->getDataLinkByAddress(network_address);
	  
	  if(DL)
	    theRouter->addInterface(DL, IPAddress(IP_address));
	  else
	    {
	      cerr << " ERROR: Data link " << network_address << " not defined." << endl;
	      if(theRouter) delete theRouter;
	      return NULL;
	    }
	}
      else if (token == "route")
	{
	  string destination_network;
	  string next_hop;
	  int metric;
	  destination_network = getString(in);
	  next_hop = getString(in);
	  metric = getInt(in);
	  
	  theRouter->addRoute(IPAddress(destination_network), IPAddress(next_hop), metric);
	}
      else
	{
	  cerr << "    Parse error on token " << token << endl;
	  if(theRouter) delete theRouter;
	  return NULL;
	}
	token = getString(in);
    }
  return theRouter;
}

/*
 * Parse a datalink from the input stream and return a pointer to it.
 * Return NULL if something goes wrong, and output debug info to cerr.
 */
DataLink * Netreader::parse_datalink(istream &in)
{
  string network_address;
  int bandwidth, delay;
  network_address = getString(in);
  bandwidth = getInt(in);
  delay = getInt(in);

 
  return new DataLink(IPAddress(network_address), bandwidth, delay);
}













